#define VERSION "1.0"

/*
    pung-1.0, a script-friendly TCP/IP server tester
    Copyright (C) 1999  James Cameron  quozl@us.netrek.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <stdio.h>		/* for printf() et. al.		*/
#include <stdlib.h>		/* for atoi()			*/
#include <unistd.h>		/* for close()			*/
#include <errno.h>		/* for errno			*/
#include <ctype.h>		/* for isdigit()		*/
#include <string.h>		/* for strerror()		*/
#include <sys/types.h>		/* for socket(), connect()	*/
#include <sys/socket.h>		/* for socket(), connect()	*/
#include <netdb.h>		/* for gethostbyname()		*/
#include <netinet/in.h>		/* for inet_addr(), htons()	*/
#include <arpa/inet.h>		/* for inet_addr()		*/

#define MAXWIDTH 128

static void usage()
{
  fprintf(stderr, 
	  "Pung version " VERSION ", Copyright (C) 1999 James Cameron\n"
	  "Pung comes with ABSOLUTELY NO WARRANTY; for details see source.\n"
	  "This is free software, and you are welcome to redistribute it\n"
	  "under certain conditions; see source for details.\n\n" );
  
  fprintf(stderr, 
	  "Usage: pung host port\n"
	  "\n"
	  "Attempts a TCP/IP connection only to the host on the given port,\n"
	  "and displays the result on standard output.  No data is\n"
	  "transferred.  Returns a status code;\n"
	  "\n"
	  "	 0 = Connection accepted\n"
	  "	 1 = Non-existent host\n"
	  "	 2 = Non-existent service\n"
	  "	 3 = connect() call failed\n"
	  "\n"
	  "Pung is intended for use in automated testing of TCP/IP servers.\n"
	  "The host and port arguments may be alphabetic or numeric.\n"
	  "\n"
	  "Examples:	pung localhost telnet\n"
	  "		pung 127.0.0.1 23\n"
	  );
}

int main(int argc, char *argv[])
{
  char *host;			/* pointer to host name arg	*/
  char *port;			/* pointer to port name arg	*/
  int fd;			/* socket file descriptor	*/
  char prefix[MAXWIDTH];	/* output text prefix buffer	*/
  struct hostent *hep, hes;	/* for gethostbyname()		*/
  struct servent *sep;		/* for getservbyname()		*/
  struct sockaddr_in name;	/* for connect()		*/

  if (argc < 3) {
    usage();
    return 10;
  }

  host = argv[1];
  port = argv[2];

  if (snprintf(prefix, MAXWIDTH, "tcp://%s:%s/", argv[1], argv[2]) == -1) {
    fprintf(stderr, "pung: arguments too long\n");
    return 11;
  }

  if ((fd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    perror("pung: socket");
    return 20;
  }

  hep = gethostbyname(host);
  if (hep == NULL) {
    name.sin_addr.s_addr = inet_addr(host);
    if (name.sin_addr.s_addr == INADDR_NONE) {
      printf("%-40s Non-existent domain\n", prefix);
      return 1;
    }
  } else {
    hes = *hep;
    name.sin_addr = *((struct in_addr *) hes.h_addr);
  }

  sep = getservbyname(port, "tcp");
  if (sep == NULL) {
    if (!isdigit(port[0])) {
      printf("%-40s Non-existent service\n", prefix);
      return 2;
    }
    name.sin_port = htons(atoi(port));
  } else {
    name.sin_port = sep->s_port;
  }

  name.sin_family = AF_INET;

  if (connect(fd, &name, sizeof(name)) != 0) {
    printf("%-40s %s\n", prefix, strerror(errno));
    return 3;
  }

  if (shutdown(fd, 2) == -1) {
    perror("pung: shutdown");
    return 21;
  }

  if(close(fd)) {
    perror("pung: close");
    return 22;
  }

  printf("%-40s Connection accepted\n", prefix);
  return 0;
}
